@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1999-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1999-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -36,6 +36,7 @@ static NV_STATUS   nv_acpi_extract_package (const union acpi_object *, void *, N
 static NV_STATUS   nv_acpi_extract_object  (const union acpi_object *, void *, NvU32, NvU32 *);
 
 static void        nv_acpi_powersource_hotplug_event(acpi_handle, u32, void *);
+static void        nv_acpi_nvpcf_event     (acpi_handle, u32, void *);
 static acpi_status nv_acpi_find_methods    (acpi_handle, u32, void *, void **);
 static NV_STATUS   nv_acpi_nvif_method     (NvU32, NvU32, void *, NvU16, NvU32 *, void *, NvU16 *);
 
@@ -65,66 +66,49 @@ static NvBool battery_present = NV_FALSE;
 #define ACPI_VIDEO_CLASS    "video"
 #endif
 
-static int nv_acpi_get_device_handle(nv_state_t *nv, acpi_handle *dev_handle)
-{
-    nv_linux_state_t *nvl = NV_GET_NVL_FROM_NV_STATE(nv);
+/* Maximum size of ACPI _DSM method's 4th argument */
+#define NV_MAX_ACPI_DSM_PARAM_SIZE     1024
 
-#if defined(DEVICE_ACPI_HANDLE)
-    *dev_handle = DEVICE_ACPI_HANDLE(nvl->dev);
-    return NV_TRUE;
-#elif defined (ACPI_HANDLE)
-    *dev_handle = ACPI_HANDLE(nvl->dev);
-    return NV_TRUE;
-#else
-    return NV_FALSE;
-#endif
-}
+// Used for NVPCF event handling
+static acpi_handle nvpcf_handle = NULL;
+static acpi_handle nvpcf_device_handle = NULL;
+static nv_acpi_t  *nvpcf_nv_acpi_object = NULL;
 
-static int nv_acpi_notify(struct notifier_block *nb,
-                        unsigned long val, void *data)
+#define ACPI_NVPCF_EVENT_CHANGE    0xC0
+
+/*
+ * This callback will be invoked by the acpi_notifier_call_chain()
+ */
+static int nv_acpi_notifier_call_chain_handler(
+    struct notifier_block *nb,
+    unsigned long val,
+    void *data
+)
 {
     struct acpi_bus_event *info = data;
-    nv_stack_t *sp = NULL;
-    nv_linux_state_t *nvl = container_of(nb, nv_linux_state_t, acpi_nb);
-    nv_state_t *nv = NV_STATE_PTR(nvl);
-
-    if (!strcmp(info->device_class, ACPI_VIDEO_CLASS)) {
-        if (nv_kmem_cache_alloc_stack(&sp) == 0) {
-            /*
-             * Function to handle device specific ACPI events
-             * such as display hotplug and D-notifier events.
-             */
-            rm_acpi_notify(sp, nv, info->type);
-            nv_kmem_cache_free_stack(sp);
-        }
-        else
-            nv_printf(NV_DBG_ERRORS,
-                "NVRM: nv_acpi_notify: failed to allocate stack\n");
 
+    /*
+     * The ACPI_VIDEO_NOTIFY_PROBE will be sent for display hot-plug/unplug.
+     * This event will be received first by the acpi-video driver
+     * and then it will be notified through acpi_notifier_call_chain().
+     */
+    if (!strcmp(info->device_class, ACPI_VIDEO_CLASS) &&
+        (info->type == ACPI_VIDEO_NOTIFY_PROBE))
+    {
         /*
-         * Special case for ACPI_VIDEO_NOTIFY_PROBE event: intentionally return
-         * NOTIFY_BAD to inform acpi-video to stop generating keypresses for
-         * this event.
+         * Intentionally return NOTIFY_BAD to inform acpi-video to stop
+         * generating keypresses for this event. The default behavior in the
+         * acpi-video driver for an ACPI_VIDEO_NOTIFY_PROBE, is to send a
+         * KEY_SWITCHVIDEOMODE evdev event, which causes the desktop settings
+         * daemons like gnome-setting-daemon to switch mode and this impacts
+         * the notebooks having external HDMI connected.
          */
-        if (info->type == ACPI_VIDEO_NOTIFY_PROBE) {
-            return NOTIFY_BAD;
-        }
+        return NOTIFY_BAD;
     }
 
     return NOTIFY_DONE;
 }
 
-void nv_acpi_register_notifier(nv_linux_state_t *nvl)
-{
-    nvl->acpi_nb.notifier_call = nv_acpi_notify;
-    register_acpi_notifier(&nvl->acpi_nb);
-}
-
-void nv_acpi_unregister_notifier(nv_linux_state_t *nvl)
-{
-    unregister_acpi_notifier(&nvl->acpi_nb);
-}
-
 NV_STATUS NV_API_CALL nv_acpi_get_powersource(NvU32 *ac_plugged)
 {
     unsigned long long val;
@@ -149,6 +133,7 @@ NV_STATUS NV_API_CALL nv_acpi_get_powersource(NvU32 *ac_plugged)
     return NV_OK;
 }
 
+#define ACPI_POWER_SOURCE_BUS_CHANGE_EVENT   0x00
 #define ACPI_POWER_SOURCE_CHANGE_EVENT       0x80
 static void nv_acpi_powersource_hotplug_event(acpi_handle handle, u32 event_type, void *data)
 {
@@ -159,20 +144,39 @@ static void nv_acpi_powersource_hotplug_event(acpi_handle handle, u32 event_type
     nv_acpi_t *pNvAcpiObject = data;
     u32 ac_plugged = 0;
 
-    if (event_type == ACPI_POWER_SOURCE_CHANGE_EVENT)
+    if (event_type == ACPI_POWER_SOURCE_CHANGE_EVENT || event_type == ACPI_POWER_SOURCE_BUS_CHANGE_EVENT)
     {
         if (nv_acpi_get_powersource(&ac_plugged) != NV_OK)
             return;
 
-        rm_system_event(pNvAcpiObject->sp, NV_SYSTEM_ACPI_BATTERY_POWER_EVENT, !ac_plugged);
+        rm_power_source_change_event(pNvAcpiObject->sp, !ac_plugged);
     }
 }
+
+static void nv_acpi_nvpcf_event(acpi_handle handle, u32 event_type, void *data)
+{
+    nv_acpi_t *pNvAcpiObject = data;
+
+    if (event_type == ACPI_NVPCF_EVENT_CHANGE)
+    {
+        rm_acpi_nvpcf_notify(pNvAcpiObject->sp);
+    }
+    else
+    {
+        nv_printf(NV_DBG_INFO,"NVRM: %s: NVPCF event 0x%x is not supported\n", __FUNCTION__, event_type);
+    }
+}
+
 /*
  * End of ACPI event handler functions
  */
 
 /* Do the necessary allocations and install notifier "handler" on the device-node "device" */
-static nv_acpi_t* nv_install_notifier(struct acpi_handle *handle, acpi_notify_handler handler)
+static nv_acpi_t* nv_install_notifier(
+    struct acpi_handle *handle,
+    acpi_notify_handler handler,
+    void               *notifier_data
+)
 {
     nvidia_stack_t *sp = NULL;
     nv_acpi_t *pNvAcpiObject = NULL;
@@ -196,9 +200,18 @@ static nv_acpi_t* nv_install_notifier(struct acpi_handle *handle, acpi_notify_ha
     // store a handle reference in our object
     pNvAcpiObject->handle = handle;
     pNvAcpiObject->sp = sp;
+    pNvAcpiObject->notifier_data = notifier_data;
 
-    status = acpi_install_notify_handler(handle, ACPI_DEVICE_NOTIFY,
-              handler, pNvAcpiObject);
+    if (handle == psr_device_handle)
+    {
+        status = acpi_install_notify_handler(handle, ACPI_ALL_NOTIFY,
+                  handler, pNvAcpiObject);
+    }
+    else
+    {
+        status = acpi_install_notify_handler(handle, ACPI_DEVICE_NOTIFY,
+                  handler, pNvAcpiObject);
+    }
     if (!ACPI_FAILURE(status))
     {
         pNvAcpiObject->notify_handler_installed = 1;
@@ -221,7 +234,14 @@ static void nv_uninstall_notifier(nv_acpi_t *pNvAcpiObject, acpi_notify_handler
 
     if (pNvAcpiObject && pNvAcpiObject->notify_handler_installed)
     {
-        status = acpi_remove_notify_handler(pNvAcpiObject->handle, ACPI_DEVICE_NOTIFY, handler);
+        if (pNvAcpiObject->handle == psr_device_handle)
+        {
+            status = acpi_remove_notify_handler(pNvAcpiObject->handle, ACPI_ALL_NOTIFY, handler);
+        }
+        else
+        {
+            status = acpi_remove_notify_handler(pNvAcpiObject->handle, ACPI_DEVICE_NOTIFY, handler);
+        }
         if (ACPI_FAILURE(status))
         {
             nv_printf(NV_DBG_INFO,
@@ -237,6 +257,48 @@ static void nv_uninstall_notifier(nv_acpi_t *pNvAcpiObject, acpi_notify_handler
     return;
 }
 
+static void nv_acpi_notify_event(acpi_handle handle, u32 event_type, void *data)
+{
+    nv_acpi_t  *pNvAcpiObject = data;
+    nv_state_t *nvl = pNvAcpiObject->notifier_data;
+
+    /*
+     * Function to handle device specific ACPI events such as display hotplug,
+     * GPS and D-notifier events.
+     */
+    rm_acpi_notify(pNvAcpiObject->sp, NV_STATE_PTR(nvl), event_type);
+}
+
+void nv_acpi_register_notifier(nv_linux_state_t *nvl)
+{
+    acpi_handle dev_handle  = ACPI_HANDLE(nvl->dev);
+
+    /* Install the ACPI notifier corresponding to dGPU ACPI device. */
+    if ((nvl->nv_acpi_object == NULL) &&
+        (dev_handle != NULL))
+    {
+        nvl->nv_acpi_object = nv_install_notifier(dev_handle, nv_acpi_notify_event, nvl);
+        if (nvl->nv_acpi_object == NULL)
+        {
+            nv_printf(NV_DBG_ERRORS,
+                "NVRM: nv_acpi_register_notifier: failed to install notifier\n");
+        }
+    }
+
+    nvl->acpi_nb.notifier_call = nv_acpi_notifier_call_chain_handler;
+    register_acpi_notifier(&nvl->acpi_nb);
+}
+
+void nv_acpi_unregister_notifier(nv_linux_state_t *nvl)
+{
+    unregister_acpi_notifier(&nvl->acpi_nb);
+    if (nvl->nv_acpi_object != NULL)
+    {
+        nv_uninstall_notifier(nvl->nv_acpi_object, nv_acpi_notify_event);
+        nvl->nv_acpi_object = NULL;
+    }
+}
+
 /*
  * acpi methods init function.
  * check if the NVIF, _DSM and WMMX methods are present in the acpi namespace.
@@ -268,10 +330,15 @@ void NV_API_CALL nv_acpi_methods_init(NvU32 *handlesPresent)
         // devices
         if (psr_nv_acpi_object == NULL)
         {
-            psr_nv_acpi_object = nv_install_notifier(psr_device_handle, nv_acpi_powersource_hotplug_event);
+            psr_nv_acpi_object = nv_install_notifier(psr_device_handle, nv_acpi_powersource_hotplug_event, NULL);
         }
     }
 
+    if (nvpcf_handle && (nvpcf_nv_acpi_object == NULL))
+    {
+        nvpcf_nv_acpi_object = nv_install_notifier(nvpcf_device_handle, nv_acpi_nvpcf_event, NULL);
+    }
+
     return;
 }
 
@@ -300,6 +367,12 @@ acpi_status nv_acpi_find_methods(
         psr_device_handle = handle;
     }
 
+    if (!acpi_get_handle(handle, "NPCF", &method_handle))
+    {
+        nvpcf_handle = method_handle;
+        nvpcf_device_handle = handle;
+    }
+
     return 0;
 }
 
@@ -316,6 +389,15 @@ void NV_API_CALL nv_acpi_methods_uninit(void)
         psr_device_handle = NULL;
         psr_nv_acpi_object = NULL;
     }
+
+    if (nvpcf_nv_acpi_object != NULL)
+    {
+        nv_uninstall_notifier(nvpcf_nv_acpi_object, nv_acpi_nvpcf_event);
+
+        nvpcf_handle = NULL;
+        nvpcf_device_handle = NULL;
+        nvpcf_nv_acpi_object = NULL;
+    }
 }
 
 static NV_STATUS nv_acpi_extract_integer(
@@ -578,64 +660,36 @@ static NV_STATUS nv_acpi_nvif_method(
     return NV_OK;
 }
 
-#define MAX_INPUT_PARAM_SIZE     1024
-/*
- * This function executes a _DSM ACPI method.
- */
-NV_STATUS NV_API_CALL nv_acpi_dsm_method(
-    nv_state_t *nv,
-    NvU8  *pAcpiDsmGuid,
-    NvU32 acpiDsmRev,
-    NvBool acpiNvpcfDsmFunction,
-    NvU32 acpiDsmSubFunction,
-    void  *pInParams,
-    NvU16 inParamSize,
-    NvU32 *outStatus,
-    void  *pOutData,
-    NvU16 *pSize
+static NV_STATUS nv_acpi_evaluate_dsm_method(
+    acpi_handle   dev_handle,
+    NvU8         *pathname,
+    NvU8         *pAcpiDsmGuid,
+    NvU32         acpiDsmRev,
+    NvU32         acpiDsmSubFunction,
+    void         *arg3,
+    NvU16         arg3Size,
+    NvBool        bArg3Integer,
+    NvU32        *outStatus,
+    void         *pOutData,
+    NvU16        *pSize
 )
 {
-    NV_STATUS status = NV_ERR_OPERATING_SYSTEM;
-    acpi_status acpi_status;
+    NV_STATUS rmStatus = NV_OK;
+    acpi_status status;
     struct acpi_object_list input;
     union acpi_object *dsm = NULL;
     struct acpi_buffer output = { ACPI_ALLOCATE_BUFFER, NULL };
     union acpi_object dsm_params[4];
-    NvU8 *argument3 = NULL;
     NvU32 data_size;
-    acpi_handle dev_handle  = NULL;
-
-    if (!nv_acpi_get_device_handle(nv, &dev_handle))
-        return NV_ERR_NOT_SUPPORTED;
-
-    if (!dev_handle)
-        return NV_ERR_INVALID_ARGUMENT;
-
-    if ((!pInParams) || (inParamSize > MAX_INPUT_PARAM_SIZE) || (!pOutData) || (!pSize))
-    {
-        nv_printf(NV_DBG_INFO,
-                  "NVRM: %s: invalid argument(s)!\n", __FUNCTION__);
-        return NV_ERR_INVALID_ARGUMENT;
-    }
 
     if (!NV_MAY_SLEEP())
     {
 #if defined(DEBUG)
-        nv_printf(NV_DBG_INFO,
-                  "NVRM: %s: invalid argument(s)!\n", __FUNCTION__);
+        nv_printf(NV_DBG_ERRORS, "NVRM: %s: invalid context!\n", __FUNCTION__);
 #endif
         return NV_ERR_NOT_SUPPORTED;
     }
 
-    status = os_alloc_mem((void **)&argument3, inParamSize);
-    if (status != NV_OK)
-        return status;
-
-    //
-    // dsm_params[0].buffer.pointer and dsm_params[1].integer.value set in
-    // switch below based on acpiDsmFunction
-    //
-
     dsm_params[0].buffer.type    = ACPI_TYPE_BUFFER;
     dsm_params[0].buffer.length  = 0x10;
     dsm_params[0].buffer.pointer = pAcpiDsmGuid;
@@ -646,35 +700,28 @@ NV_STATUS NV_API_CALL nv_acpi_dsm_method(
     dsm_params[2].integer.type   = ACPI_TYPE_INTEGER;
     dsm_params[2].integer.value  = acpiDsmSubFunction;
 
-    dsm_params[3].buffer.type    = ACPI_TYPE_BUFFER;
-    dsm_params[3].buffer.length  = inParamSize;
-    memcpy(argument3, pInParams, dsm_params[3].buffer.length);
-    dsm_params[3].buffer.pointer = argument3;
+    if (bArg3Integer)
+    {
+        dsm_params[3].integer.type  = ACPI_TYPE_INTEGER;
+        dsm_params[3].integer.value = *((NvU32 *)arg3);
+    }
+    else
+    {
+         dsm_params[3].buffer.type    = ACPI_TYPE_BUFFER;
+         dsm_params[3].buffer.length  = arg3Size;
+         dsm_params[3].buffer.pointer = arg3;
+    }
 
     // parameters for dsm calls (GUID, rev, subfunction, data)
     input.count = 4;
     input.pointer = dsm_params;
 
-    if (acpiNvpcfDsmFunction)
-    {
-        //
-        // acpi_evaluate_object() can operate with either valid object pathname or
-        // valid object handle. For NVPCF DSM function, use valid pathname as we do
-        // not have device handle for NVPCF device
-        //
-        dev_handle = NULL;
-        acpi_status = acpi_evaluate_object(dev_handle, "\\_SB.NPCF._DSM", &input, &output);
-    }
-    else
-    {
-        acpi_status = acpi_evaluate_object(dev_handle, "_DSM", &input, &output);
-    }
-
-    if (ACPI_FAILURE(acpi_status))
+    status = acpi_evaluate_object(dev_handle, pathname, &input, &output);
+    if (ACPI_FAILURE(status))
     {
         nv_printf(NV_DBG_INFO,
               "NVRM: %s: failed to evaluate _DSM method!\n", __FUNCTION__);
-        goto exit;
+        return NV_ERR_OPERATING_SYSTEM;
     }
 
     dsm = output.pointer;
@@ -688,20 +735,80 @@ NV_STATUS NV_API_CALL nv_acpi_dsm_method(
                          dsm->buffer.pointer[0];
         }
 
-        status = nv_acpi_extract_object(dsm, pOutData, *pSize, &data_size);
+        rmStatus = nv_acpi_extract_object(dsm, pOutData, *pSize, &data_size);
         *pSize = data_size;
 
         kfree(output.pointer);
     }
-    if (status != NV_OK)
+    else
+    {
+        *pSize = 0;
+    }
+
+    if (rmStatus != NV_OK)
     {
         nv_printf(NV_DBG_ERRORS,
                   "NVRM: %s: DSM data invalid!\n", __FUNCTION__);
     }
 
-exit:
+    return rmStatus;
+}
+
+/*
+ * This function executes a _DSM ACPI method.
+ */
+NV_STATUS NV_API_CALL nv_acpi_dsm_method(
+    nv_state_t  *nv,
+    NvU8        *pAcpiDsmGuid,
+    NvU32        acpiDsmRev,
+    NvBool       acpiNvpcfDsmFunction,
+    NvU32        acpiDsmSubFunction,
+    void        *pInParams,
+    NvU16        inParamSize,
+    NvU32       *outStatus,
+    void        *pOutData,
+    NvU16       *pSize
+)
+{
+    NV_STATUS rmStatus     = NV_ERR_OPERATING_SYSTEM;
+    NvU8 *argument3        = NULL;
+    nv_linux_state_t *nvl  = NV_GET_NVL_FROM_NV_STATE(nv);
+    acpi_handle dev_handle = ACPI_HANDLE(nvl->dev);
+    NvU8         *pathname = "_DSM";
+
+    if (!dev_handle)
+        return NV_ERR_INVALID_ARGUMENT;
+
+    if ((!pInParams) || (inParamSize > NV_MAX_ACPI_DSM_PARAM_SIZE) || (!pOutData) || (!pSize))
+    {
+        nv_printf(NV_DBG_INFO,
+                  "NVRM: %s: invalid argument(s)!\n", __FUNCTION__);
+        return NV_ERR_INVALID_ARGUMENT;
+    }
+
+    rmStatus = os_alloc_mem((void **)&argument3, inParamSize);
+    if (rmStatus != NV_OK)
+        return rmStatus;
+
+    memcpy(argument3, pInParams, inParamSize);
+
+    if (acpiNvpcfDsmFunction)
+    {
+        //
+        // acpi_evaluate_object() can operate with either valid object pathname or
+        // valid object handle. For NVPCF DSM function, use valid pathname as we do
+        // not have device handle for NVPCF device
+        //
+        dev_handle = NULL;
+        pathname   = "\\_SB.NPCF._DSM";
+    }
+
+    rmStatus = nv_acpi_evaluate_dsm_method(dev_handle, pathname, pAcpiDsmGuid, acpiDsmRev,
+                                           acpiDsmSubFunction, argument3, inParamSize,
+                                           NV_FALSE, NULL, pOutData, pSize);
+
     os_free_mem(argument3);
-    return status;
+    return rmStatus;
 }
 
 /*
@@ -715,28 +822,16 @@ NV_STATUS NV_API_CALL nv_acpi_ddc_method(
 )
 {
     acpi_status status;
-    struct acpi_device *device = NULL;
     union acpi_object *ddc = NULL;
-    struct list_head *node, *next;
     NvU32 i, largestEdidSize;
-    acpi_handle dev_handle  = NULL;
+    nv_linux_state_t *nvl  = NV_GET_NVL_FROM_NV_STATE(nv);
+    acpi_handle dev_handle = ACPI_HANDLE(nvl->dev);
     acpi_handle lcd_dev_handle  = NULL;
-
-    if (!nv_acpi_get_device_handle(nv, &dev_handle))
-        return NV_ERR_NOT_SUPPORTED;
+    acpi_handle handle = NULL;
 
     if (!dev_handle)
         return NV_ERR_INVALID_ARGUMENT;
 
-#if defined(NV_ACPI_BUS_GET_DEVICE_PRESENT)
-    status = acpi_bus_get_device(dev_handle, &device);
-#else
-    return NV_ERR_NOT_SUPPORTED;
-#endif
-
-    if (ACPI_FAILURE(status) || !device)
-        return NV_ERR_INVALID_ARGUMENT;
-
     if (!NV_MAY_SLEEP())
     {
 #if defined(DEBUG)
@@ -747,16 +842,16 @@ NV_STATUS NV_API_CALL nv_acpi_ddc_method(
         return NV_ERR_NOT_SUPPORTED;
     }
 
-    list_for_each_safe(node, next, &device->children)
+    while (lcd_dev_handle == NULL)
     {
         unsigned long long device_id = 0;
-        struct acpi_device *dev =
-            list_entry(node, struct acpi_device, node);
 
-        if (!dev)
-            continue;
+        status = acpi_get_next_object(ACPI_TYPE_DEVICE, dev_handle,
+                                      handle, &handle);
+        if (ACPI_FAILURE(status) || (handle == NULL))
+            break;
 
-        status = acpi_evaluate_integer(dev->handle, "_ADR", NULL, &device_id);
+        status = acpi_evaluate_integer(handle, "_ADR", NULL, &device_id);
         if (ACPI_FAILURE(status))
             /* Couldnt query device_id for this device */
             continue;
@@ -766,16 +861,13 @@ NV_STATUS NV_API_CALL nv_acpi_ddc_method(
             case 0x0118:
             case 0x0400:
             case 0xA420:
-                lcd_dev_handle = dev->handle;
-                nv_printf(NV_DBG_INFO, "NVRM: %s Found LCD: %x\n",
+                lcd_dev_handle = handle;
+                nv_printf(NV_DBG_INFO, "NVRM: %s Found LCD: %llx\n",
                           __FUNCTION__, device_id);
                 break;
             default:
                 break;
         }
-
-        if (lcd_dev_handle != NULL)
-            break;
     }
 
     if (lcd_dev_handle == NULL)
@@ -849,12 +941,10 @@ NV_STATUS NV_API_CALL nv_acpi_rom_method(
     union acpi_object *rom;
     union acpi_object rom_arg[2];
     struct acpi_object_list input = { 2, rom_arg };
-    acpi_handle dev_handle  = NULL;
+    nv_linux_state_t *nvl  = NV_GET_NVL_FROM_NV_STATE(nv);
+    acpi_handle dev_handle = ACPI_HANDLE(nvl->dev);
     uint32_t offset, length;
 
-    if (!nv_acpi_get_device_handle(nv, &dev_handle))
-        return NV_ERR_NOT_SUPPORTED;
-
     if (!dev_handle)
         return NV_ERR_INVALID_ARGUMENT;
 
@@ -916,12 +1006,10 @@ NV_STATUS NV_API_CALL nv_acpi_dod_method(
     acpi_status status;
     struct acpi_buffer output = { ACPI_ALLOCATE_BUFFER, NULL };
     union acpi_object *dod;
-    acpi_handle dev_handle = NULL;
+    nv_linux_state_t *nvl  = NV_GET_NVL_FROM_NV_STATE(nv);
+    acpi_handle dev_handle = ACPI_HANDLE(nvl->dev);
     NvU32 i, count = (*pSize / sizeof(NvU32));
 
-    if (!nv_acpi_get_device_handle(nv, &dev_handle))
-        return NV_ERR_NOT_SUPPORTED;
-
     if (!dev_handle)
         return NV_ERR_INVALID_ARGUMENT;
 
@@ -993,7 +1081,10 @@ static NV_STATUS  nv_acpi_wmmx_method(
     union acpi_object mmx_params[3];
 
     if (!wmmx_handle)
+    {
+        *outDataSize = 0;
         return NV_ERR_NOT_SUPPORTED;
+    }
 
     if (!NV_MAY_SLEEP())
     {
@@ -1060,17 +1151,11 @@ NvBool nv_acpi_power_resource_method_present(
     struct pci_dev *pdev
 )
 {
-    acpi_handle handle = NULL;
+    acpi_handle handle = ACPI_HANDLE(&pdev->dev);
     struct acpi_buffer buf = { ACPI_ALLOCATE_BUFFER, NULL };
     union acpi_object *object_package, *object_reference;
     acpi_status status;
 
-#if defined(DEVICE_ACPI_HANDLE)
-    handle = DEVICE_ACPI_HANDLE(&pdev->dev);
-#elif defined (ACPI_HANDLE)
-    handle = ACPI_HANDLE(&pdev->dev);
-#endif
-
     if (!handle)
         return NV_FALSE;
 
@@ -1125,15 +1210,15 @@ NV_STATUS NV_API_CALL nv_acpi_mux_method(
 )
 {
     acpi_status status;
-    struct acpi_device *device    = NULL;
     struct acpi_buffer output     = { ACPI_ALLOCATE_BUFFER, NULL };
     union acpi_object *mux        = NULL;
     union acpi_object mux_arg     = { ACPI_TYPE_INTEGER };
     struct acpi_object_list input = { 1, &mux_arg };
-    acpi_handle dev_handle        = NULL;
+    nv_linux_state_t *nvl         = NV_GET_NVL_FROM_NV_STATE(nv);
+    acpi_handle dev_handle        = ACPI_HANDLE(nvl->dev);
     acpi_handle mux_dev_handle    = NULL;
+    acpi_handle handle            = NULL;
     unsigned long long device_id  = 0;
-    struct list_head *node, *next;
 
     if ((strcmp(pMethodName, "MXDS") != 0)
         && (strcmp(pMethodName, "MXDM") != 0))
@@ -1148,22 +1233,9 @@ NV_STATUS NV_API_CALL nv_acpi_mux_method(
                   __FUNCTION__, pMethodName);
     }
 
-    if (!nv_acpi_get_device_handle(nv, &dev_handle))
-        return NV_ERR_NOT_SUPPORTED;
-
     if (!dev_handle)
         return NV_ERR_INVALID_ARGUMENT;
 
-#if defined(NV_ACPI_BUS_GET_DEVICE_PRESENT)
-    status = acpi_bus_get_device(dev_handle, &device);
-#else
-    return NV_ERR_NOT_SUPPORTED;
-#endif
-
-
-    if (ACPI_FAILURE(status) || !device)
-        return NV_ERR_INVALID_ARGUMENT;
-
     if (!NV_MAY_SLEEP())
     {
 #if defined(DEBUG)
@@ -1172,23 +1244,16 @@ NV_STATUS NV_API_CALL nv_acpi_mux_method(
         return NV_ERR_NOT_SUPPORTED;
     }
 
-    list_for_each_safe(node, next, &device->children)
+    while (mux_dev_handle == NULL)
     {
-        struct acpi_device *dev = list_entry(node, struct acpi_device, node);
-
-        if (!dev)
-            continue;
-
-        status = acpi_evaluate_integer(dev->handle, "_ADR", NULL, &device_id);
-        if (ACPI_FAILURE(status))
-            /* Could not query device_id for this device */
-            continue;
-
-        if (device_id == muxAcpiId)
-        {
-            mux_dev_handle = dev->handle;
+        status = acpi_get_next_object(ACPI_TYPE_DEVICE, dev_handle,
+                                      handle, &handle);
+        if (ACPI_FAILURE(status) || (handle == NULL))
             break;
-        }
+
+        status = acpi_evaluate_integer(handle, "_ADR", NULL, &device_id);
+        if (ACPI_SUCCESS(status) && (device_id == muxAcpiId))
+            mux_dev_handle = handle;
     }
 
     if (mux_dev_handle == NULL)
@@ -1333,6 +1398,34 @@ NvBool NV_API_CALL nv_acpi_is_battery_present(void)
     return NV_FALSE;
 }
 
+NV_STATUS NV_API_CALL nv_acpi_d3cold_dsm_for_upstream_port(
+    nv_state_t *nv,
+    NvU8       *pAcpiDsmGuid,
+    NvU32       acpiDsmRev,
+    NvU32       acpiDsmSubFunction,
+    NvU32      *data
+)
+{
+    NV_STATUS rmStatus = NV_ERR_OPERATING_SYSTEM;
+    nv_linux_state_t *nvl  = NV_GET_NVL_FROM_NV_STATE(nv);
+    acpi_handle dev_handle = ACPI_HANDLE(nvl->dev->parent);
+    NvU32 outData     = 0;
+    NvU16 outDatasize = sizeof(NvU32);
+    NvU16 inParamSize = sizeof(NvU32);
+
+    if (!dev_handle)
+        return NV_ERR_INVALID_ARGUMENT;
+
+    rmStatus = nv_acpi_evaluate_dsm_method(dev_handle, "_DSM", pAcpiDsmGuid, acpiDsmRev,
+                                           acpiDsmSubFunction, data, inParamSize, NV_TRUE,
+                                           NULL, &outData, &outDatasize);
+
+    if (rmStatus == NV_OK)
+        *data = outData;
+
+    return rmStatus;
+}
+
 #else // NV_LINUX_ACPI_EVENTS_SUPPORTED
 
 void NV_API_CALL nv_acpi_methods_init(NvU32 *handlePresent)
@@ -1375,6 +1468,17 @@ NV_STATUS NV_API_CALL nv_acpi_dsm_method(
     return NV_ERR_NOT_SUPPORTED;
 }
 
+NV_STATUS NV_API_CALL nv_acpi_d3cold_dsm_for_upstream_port(
+    nv_state_t *nv,
+    NvU8       *pAcpiDsmGuid,
+    NvU32       acpiDsmRev,
+    NvU32       acpiDsmSubFunction,
+    NvU32      *data
+)
+{
+    return NV_ERR_NOT_SUPPORTED;
+}
+
 NV_STATUS NV_API_CALL nv_acpi_ddc_method(
     nv_state_t *nv,
     void *pEdidBuffer,
